#ifndef VERSATILE_TILE_EDITOR_HEADER_INCLUDED
#define VERSATILE_TILE_EDITOR_HEADER_INCLUDED

#include <windows.h>
#include "mydibobj3.h"
#include "nnsys.h"
#include "myeinput.h"
#include "keycode.h"
#include "TypedFunction.h"
#include "undo.h"

class VersatileTileEditor
{
public:
	VersatileTileEditor( unsigned char *pucData , int iDataWidth , int iDataHeight , int iTileW , int iTileH , int iUndoBuffer )
	{
		pucClip = NULL ;
		iTilePaletteX = iTilePaletteY = 0 ;
		iSFCTile = -1 ;
		strClipFileName = _T("") ;
		Clear( pucData , iDataWidth , iDataHeight , iTileW , iTileH ) ;
		SetSelectedTile( 0 ) ;
		pUndo = NULL ;
		if( iUndoBuffer > 0 )
		{
			pUndo = new Undo( pucData , iDataWidth*iDataHeight , iUndoBuffer ) ;
		}
	}
	~VersatileTileEditor()
	{
		DeletepucClip() ;
		if( pUndo )
		{
			delete pUndo ;
			pUndo = NULL ;
		}
	}
	void Clear( unsigned char *pucData , int iDataWidth , int iDataHeight , int iTileW , int iTileH )
	{
		this->pucData     = pucData ;
		this->iDataWidth  = iDataWidth ;
		this->iDataHeight = iDataHeight ;
		SetPosition( 0 , 0 ) ;
		SetTileSize( iTileW , iTileH ) ;
		SetWindow( 0 , 0 , iDataWidth , iDataHeight ) ;
		iDragging = 0 ;
		iSelectedX0 = iSelectedX1 = 0 ;
		iSelectedY0 = iSelectedY1 = 0 ;
		DeletepucClip() ;
	}
	void SetSFCTile( int iSFCTile )
	{
		this->iSFCTile = iSFCTile ;
	}
	void SetPosition( int iOrgX , int iOrgY )
	{
		this->iOrgX = iOrgX ;
		this->iOrgY = iOrgY ;
	}
	void SetTileSize( int iTileW , int iTileH )
	{
		this->iTileW = iTileW ;
		this->iTileH = iTileH ;
	}
	void SetWindow( int iWindowX , int iWindowY , int iWindowW , int iWindowH )
	{
		this->iWindowX = this->iWindowY = 0 ;
		this->iWindowW = this->iWindowH = 0 ;
		if( iWindowX < 0 ){ iWindowW+=iWindowX ; iWindowX = 0 ; }
		if( iWindowX >= iDataWidth ){ return ; }
		if( iWindowY < 0 ){ iWindowH+=iWindowY ; iWindowY = 0 ; }
		if( iWindowY >= iDataHeight ){ return ; }

		if( iWindowW < 0 ){ return ; }
		if( iWindowX+iWindowW > iDataWidth ){ iWindowW = iDataWidth - iWindowX ; }
		if( iWindowH < 0 ){ return ; }
		if( iWindowY+iWindowH > iDataHeight ){ iWindowH = iDataHeight - iWindowY ; }

		this->iWindowX = iWindowX ;
		this->iWindowY = iWindowY ;
		this->iWindowW = iWindowW ;
		this->iWindowH = iWindowH ;
	}
	void SetSelectedTile( int iSelectedTile )
	{
		this->iSelectedTile = iSelectedTile ;
	}
	void SetTilePalette( int iX , int iY )
	{
		this->iTilePaletteX = iX ;
		this->iTilePaletteY = iY ;
	}
	void SetClipFileName( tstring strClipFileName )
	{
		this->strClipFileName = strClipFileName ;
	}
	int GetData( int iX , int iY )
	{
		if( !IsPositionInData( iX , iY ) )
			{ return -1 ; }
		return pucData[ GetOffset( iX,iY ) ] ;
	}
	void SetData( int iX , int iY , int iData , int iIgnoreWindow )
	{
		if( iIgnoreWindow )
		{
			if( !IsPositionInData( iX , iY ) )
				{ return ; }
		} else {
			if( !IsPositionInWindow( iX , iY ) )
				{ return ; }
		}
		pucData[ GetOffset( iX,iY ) ] = iData ;
	}
	int DoEdit() ;
	void Draw( MyDIBObj3 *pMDO , int iSFC ) ;
	void LoadClipFile( LPCTSTR filename ) ;
	void GetUndoCounter( int *piCurrent , int *piTimes )
	{
		if( pUndo )
			{pUndo->GetCounter( piCurrent , piTimes ) ;	}
	}
	void ResetUndo()
	{
		if( pUndo ){ pUndo->Reset(); }
	}
private:
	unsigned char *pucData ;
	int iDataWidth ;
	int iDataHeight ;
	int iWindowX , iWindowW ;
	int iWindowY , iWindowH ;
	int iOrgX , iOrgY ;
	int iTileW , iTileH ;
	int iSelectedTile ;
	int iDragging ;
	int iSelectedX0 , iSelectedY0 , iDragOrgX ;
	int iSelectedX1 , iSelectedY1 , iDragOrgY ;
	unsigned char *pucClip ;
	int iClipX , iClipY ;
	int iClipW , iClipH ;
	tstring strClipFileName ;
	int iTilePaletteX , iTilePaletteY ;
	int iSFCTile ;
	Undo *pUndo ;

	int DoEdit_Sub0() ;
	int DoEdit_Sub1() ;
	int DoEdit_Sub2() ;
	int SetMousePosition( int *piX , int *piY ) ;
	void SetMousePositionN( int *piX , int *piY ) ;
	int IsPositionInData( int iX , int iY )
	{
		if( iX < 0 || iX >= iDataWidth ||
			iY < 0 || iY >= iDataHeight )
			{ return 0 ; }
		return 1 ;
	}
	int IsPositionInWindow( int iX , int iY )
	{
		if( iX < iWindowX || iX >= iWindowX+iWindowW ||
			iY < iWindowY || iY >= iWindowY+iWindowH )
			{ return 0 ;}
		return 1 ;
	}
	int GetOffset( int iX , int iY )
	{
		return iX+iY*iDataWidth ;
	}
	void DeletepucClip()
	{
		if( pucClip ){ delete pucClip ; }
		pucClip = NULL ;
	}
} ;

#endif /*VERSATILE_TILE_EDITOR_HEADER_INCLUDED*/
