#ifndef _6502_HEADER_INCLUDED
#define _6502_HEADER_INCLUDED



#ifdef _DEBUG
#define		MEMORY_CHECK
#endif

#define		TRACER_ENABLED

#ifdef TRACER_ENABLED
#define		TRACE_SIZE		0x400
#endif


#define		_6502m_EXIT			0x20
#define		_6502m_BREAK		0x70
#define		_6502m_ERNO_TRIAL_OUT	0x71


typedef struct Register6502_tag
{
	BYTE a;
	BYTE x;
	BYTE y;
	BYTE s;
	BYTE p;
	WORD pc;
}Register6502;

typedef struct DataSet_tag
{
	Register6502 reg;
	BYTE *Pram;
	BYTE *Prom;
	BYTE MMC3command;
	BYTE *Pprg[4];
	BYTE prgNO[4];
	BYTE DUMMY_BYTE;
#ifdef MEMORY_CHECK
	BYTE *Pramc;
#endif
}DataSet;


class FCEmu
{
protected:
	DataSet d;
#ifdef TRACER_ENABLED
	int trace_buffer[TRACE_SIZE];
	int trace_point;
#endif
	void OprateOnce();

public :
	FCEmu();
	~FCEmu(){Release();}
	void Initialize();
	void ClearRegister();
	void Release();
	void SetRomData( BYTE *Pin){d.Prom=Pin;}
};

class FCEmuMinus : public FCEmu
{
private:

public :
	void ShowOpPos();
	char *AnyTimesOprate( int NOop );
	char *OprateUntilRTS( int limit );
	char *OprateUntilRTSwithBreak( int limit , BYTE bank , WORD ipc );
	int GetBANKPC(){
		switch(d.reg.pc/0x2000)
		{
		case 4:
			return (d.prgNO[0]<<16)|d.reg.pc;
		case 5:
			return (d.prgNO[1]<<16)|d.reg.pc;
		case 6:
			return (d.prgNO[2]<<16)|d.reg.pc;
		case 7:
			return (d.prgNO[3]<<16)|d.reg.pc;
		default:
			return d.reg.pc;
		}
	}
	WORD GetPC(){return d.reg.pc;}
	BYTE GetA(){return d.reg.a;}
	BYTE GetX(){return d.reg.x;}
	BYTE GetY(){return d.reg.y;}
	BYTE GetP(){return d.reg.p;}
	BYTE GetS(){return d.reg.s;}
	BYTE *GetBank(){return d.prgNO;}
	void WriteA(BYTE val){d.reg.a = val;}
	void WriteX(BYTE val){d.reg.x = val;}
	void WriteY(BYTE val){d.reg.y = val;}
	void WriteAdr(int adr , BYTE val);
	void WriteAdrW(int adr , WORD val);
	BYTE GetAdr( int adr );
	BYTE *GetRAMPointer( ){return d.Pram;}
	BYTE *GetROMPointer( ){return d.Prom;}
	void EasyInitialize( int PC , BYTE p , BYTE b0 , BYTE b1 , BYTE b2 , BYTE b3 )
	{
		d.reg.pc=PC;
		d.reg.p=p;
		if( b0!=0xFF)
			d.prgNO[0] = b0 ;
		if( b1!=0xFF)
			d.prgNO[1] = b1 ;
		if( b2!=0xFF)
			d.prgNO[2] = b2 ;
		if( b3!=0xFF)
			d.prgNO[3] = b3 ;
		d.Pprg[0] = &d.Prom[d.prgNO[0]*0x2000+0x10] ;
		d.Pprg[1] = &d.Prom[d.prgNO[1]*0x2000+0x10] ;
		d.Pprg[2] = &d.Prom[d.prgNO[2]*0x2000+0x10] ;
		d.Pprg[3] = &d.Prom[d.prgNO[3]*0x2000+0x10] ;
	};
};


#endif /*_6502_HEADER_INCLUDED*/