#ifndef FODS_P_TOKEN_HEADER_INCLUDED
#define FODS_P_TOKEN_HEADER_INCLUDED

class FODS_TOKEN
{
public:
	static const int cFExist    = 1 ;
	static const int cFSelected = 2 ;

	FODS_TOKEN() { Clear() ;}
	~FODS_TOKEN() {;}

	void Clear(){ iFlag=iX=iY=iT= 0 ;}
	void Delete(){ iFlag = 0 ;}
	void Put( int iX , int iY , int iT )
	{
		iFlag = cFExist ;
		Modify( iX , iY , iT ) ;
	}

	void Modify( int iX , int iY , int iT )
	{
		if( !IsExist() )return ;
		if( iX >= 0 )this->iX = iX ;
		if( iY >= 0 )this->iY = iY ;
		if( iT >= 0 )this->iT = iT ;
	}
	void ModifyDelta( int iX , int iY , int iT )
	{
		if( !IsExist() )return ;
		this->iX += iX ;
		this->iY += iY ;
		this->iT += iT ;
	}
	void Select()  { if( IsExist() ){ iFlag|= cFSelected; } }
	void Deselect(){ if( IsExist() ){ iFlag&=~cFSelected; } }
	void SwitchSelection(){ if( IsExist() ){ iFlag^=cFSelected; } }

	int IsExist(){ if( iFlag&cFExist )return 1 ;return 0 ; }
	int IsSelected(){ if( (iFlag&(cFExist|cFSelected))==(cFExist|cFSelected) )return 1 ;return 0 ; }
	int GetFlag(){ return iFlag ; }
	int GetX(){ return iX ; }
	int GetY(){ return iY ; }
	int GetT(){ return iT ; }
	virtual void Draw( MyDIBObj3 *pMDO , int iSFC , int iBaseX , int iBaseY ){;}
	FODS_TOKEN& operator=( const FODS_TOKEN& operand )
	{
		iFlag = operand.iFlag ;
		iX = operand.iX ;
		iY = operand.iY ;
		iT = operand.iT ;
		return *this ;
	}
protected:
	void Write16( unsigned char *pucDest , short val )
	{
		pucDest[0] = ((short)val)     &0xFF ;
		pucDest[1] = (((short)val)>>8)&0xFF ;
	}
	short Read16( unsigned char *pucSrc )
	{
		return (short)(pucSrc[0]|(pucSrc[1]<<8)) ;
	}

	int iFlag ;
	int iX ;
	int iY ;
	int iT ;
private:
} ;


class FODS_OBJ : public FODS_TOKEN
{
public:
	void SetupImage( unsigned char *pucOut )
	{
		Write16( pucOut+0 , iFlag ) ;
		Write16( pucOut+2 , iX    ) ;
		Write16( pucOut+4 , iY    ) ;
		Write16( pucOut+6 , iT    ) ;
	}
	void ApplyImage( unsigned char *pucIn )
	{
		iFlag = Read16( pucIn+0 ) ;
		iX    = Read16( pucIn+2 ) ;
		iY    = Read16( pucIn+4 ) ;
		iT    = Read16( pucIn+6 ) ;
	}
	void Draw( MyDIBObj3 *pMDO , int iSFC , int iBaseX , int iBaseY ) ;
private:
} ;

class FODS_DOOR : public FODS_TOKEN
{
public:
	void Modify( int iX , int iY , int iT , int iDestR , int iDestP , int iDestX , int iDestY )
	{
		if( !IsExist() )return ;
		FODS_TOKEN::Modify( iX , iY , iT ) ;
		if( iDestR >= 0 )this->iDestR = iDestR ;
		if( iDestP >= 0 )this->iDestP = iDestP  ;
		if( iDestX >= 0 )this->iDestX = iDestX ;
		if( iDestY >= 0 )this->iDestY = iDestY ;
	}
	int GetDestR(){ return iDestR ; }
	int GetDestP(){ return iDestP ; }
	int GetDestX(){ return iDestX ; }
	int GetDestY(){ return iDestY ; }
	void SetupImage( unsigned char *pucOut )
	{
		Write16( pucOut+0 , iFlag  ) ;
		Write16( pucOut+2 , iX     ) ;
		Write16( pucOut+4 , iY     ) ;
		Write16( pucOut+6 , iT     ) ;
		Write16( pucOut+8 , iDestR ) ;
		Write16( pucOut+10, iDestP ) ;
		Write16( pucOut+12, iDestX ) ;
		Write16( pucOut+14, iDestY ) ;
	}
	void ApplyImage( unsigned char *pucIn )
	{
		iFlag  = Read16( pucIn+0 ) ;
		iX     = Read16( pucIn+2 ) ;
		iY     = Read16( pucIn+4 ) ;
		iT     = Read16( pucIn+6 ) ;
		iDestR = Read16( pucIn+8 ) ;
		iDestP = Read16( pucIn+10) ;
		iDestX = Read16( pucIn+12) ;
		iDestY = Read16( pucIn+14) ;
	}
	void Draw( MyDIBObj3 *pMDO , int iSFC , int iBaseX , int iBaseY ) ;
	FODS_DOOR& operator=( const FODS_DOOR& operand )
	{
		FODS_TOKEN::operator=( operand ) ;
		iDestR = operand.iDestR ;
		iDestP = operand.iDestP ;
		iDestX = operand.iDestX ;
		iDestY = operand.iDestY ;
		return *this ;
	}
private:
	int iDestR ;
	int iDestP ;
	int iDestX ;
	int iDestY ;
} ;


#endif /*FODS_P_TOKEN_HEADER_INCLUDED*/

