#include <windows.h>
#include <stdio.h>
#include <assert.h>

static int Operate20Score(const BYTE *Psrc , int pos , int maxpos , bool isadjust=true);
static int Operate40Score(const BYTE *Psrc , int pos , int maxpos , bool isadjust=true);
static int Operate60Score(const BYTE *Psrc , int pos , int maxpos , bool isadjust=true);
static int Operate80Score(const BYTE *Psrc , int endpos , int length , int *refpos = NULL , bool isadjust=true);
static int OperateA0Score(const BYTE *Psrc , int endpos , int length , int *refpos = NULL , bool isadjust=true);
static int OperateC0Score(const BYTE *Psrc , int endpos , int length , int *refpos = NULL , bool isadjust=true);

static void Operate20(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos );
static void Operate40(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos );
static void Operate60(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos );
static void Operate80(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos );
static void OperateA0(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos );
static void OperateC0(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos );

static BYTE ByteReverse(BYTE iv)
{
	return
		((iv & 0x80) >> 7) |
		((iv & 0x40) >> 5) |
		((iv & 0x20) >> 3) |
		((iv & 0x10) >> 1) |
		((iv & 0x08) << 1) |
		((iv & 0x04) << 3) |
		((iv & 0x02) << 5) |
		((iv & 0x01) << 7) ;
}


int TwinkleDecode( BYTE *Pbuf , int wlimit , const BYTE *Psrc , int rlimit , int *outrp , int *outwp)
{
const BYTE *Prpos;
BYTE *Pwpos;
int rp,wp;
int noloop;
BYTE ope;
int q;
	Prpos = Psrc ;
	Pwpos = Pbuf ;
	rp = 0;
	wp = 0;
	for(;;)
	{
		if( rp>=rlimit )return 1 ;
		if(Prpos[rp] == 0xFF)break;
		if((Prpos[rp] & 0xE0) == 0xE0)
		{
			if( rp+1>=rlimit )return 1 ;
			ope = (Prpos[rp]<<3) & 0xE0;
			noloop = ((int)(Prpos[rp] & 0x3)<<8 )+Prpos[rp+1];
			rp += 2;
		}
		else
		{
			ope = Prpos[rp] & 0xE0;
			noloop = Prpos[rp] & 0x1F;
			rp++;
		}
		noloop++;

		if(wlimit!=-1 && wlimit < wp+noloop*(1+(ope==0x40)))
		{
			if(outrp)
				*outrp = rp+1;
			if(outwp)
				*outwp = wp;
			return 1;
		}


		switch(ope)
		{
		case 0x00:
			for(q=0 ; q<noloop ; q++)
			{
				if( rp>=rlimit )return 1 ;
				Pwpos[wp] = Prpos[rp];
				wp++;
				rp++;
			}
		break;
		case 0x20:
			for(q=0 ; q<noloop ; q++)
			{
				Pwpos[wp] = Prpos[rp];
				wp++;
			}
			rp++;
		break;
		case 0x40:
			{
				if( rp+1>=rlimit )return 1 ;
				BYTE tmp,tmp2;
				tmp  = Prpos[rp  ];
				tmp2 = Prpos[rp+1];
				rp+=2;
				for(q=0 ; q<noloop ; q++)
				{
					Pwpos[wp]   = tmp;
					Pwpos[wp+1] = tmp2;
					wp+=2;
				}
			}
		break;
		case 0x60:
			{
				BYTE tmp;
				if( rp>=rlimit )return 1 ;
				tmp = Prpos[rp];
				for(q=0 ; q<noloop ; q++)
				{
					Pwpos[wp] = tmp;
					wp++;
					tmp++;
				}
				rp++;
			}
		break;
		case 0xE0:
		case 0x80:
			{
				if( rp+1>=rlimit )return 1 ;
				int ref0 ;
				ref0 = Prpos[rp]*0x0100+Prpos[rp+1] ;
				if( ref0 >= wp )
				{
					if(outrp)
						*outrp = rp+1;
					if(outwp)
						*outwp = wp;
					return 1;
				}
				for(q=0 ; q<noloop ; q++)
				{
					Pwpos[wp] = Pwpos[ref0+q];
					wp++;
				}
				rp += 2;
			}
		break;
		case 0xA0:
			{
				if( rp+1>=rlimit )return 1 ;
				int ref0 ;
				ref0 = Prpos[rp]*0x0100+Prpos[rp+1] ;
				if( ref0 >= wp )
				{
					if(outrp)
						*outrp = rp+1;
					if(outwp)
						*outwp = wp;
					return 1;
				}
				for(q=0 ; q<noloop ; q++)
				{
					Pwpos[wp] = ByteReverse(Pwpos[ref0+q]);
					wp++;
				}
				rp += 2;
			}
		break;
		case 0xC0:
			{
				if( rp+1>=rlimit )return 1 ;
				int ref0 ;
				ref0 = Prpos[rp]*0x0100+Prpos[rp+1] ;
				if( ref0 >= wp )
				{
					if(outrp)
						*outrp = rp+1;
					if(outwp)
						*outwp = wp;
					return 1;
				}
				if( ref0-noloop+1 < 0 )
				{
					return 1 ;
				}
				for(q=0 ; q<noloop ; q++)
				{
					Pwpos[wp] = Pwpos[ref0-q];
					wp++;
				}
				rp += 2;
			}
		break;
		}
	}
	if(outrp)
		*outrp = rp+1;
	if(outwp)
		*outwp = wp;
	return 0;
}




//e툳kIy[gB*ScoreĂ̂́A_vZĂB
int Operate20Score(const BYTE *Psrc , int pos , int maxpos , bool isadjust)
{
int tmp;
	tmp = Psrc[pos];
	int i ;
	for(i=0 ; ; i++)
	{
		if(i+pos >= maxpos){break;}
		if(Psrc[i+pos] != tmp)break;
	}
	//iA
	if(isadjust)
		return i-2;
	return i;
}
void Operate20(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos )
{
int nowrite=Operate20Score(Psrc , *rpos , maxpos , false)-1;
int tmp=0;
	if( nowrite > 0x1F)
	{
		if( nowrite > 0x03FF)
		{
			nowrite = 0x03FF;
		}
		tmp = 1;
		Pbuffer[*wpos]     = 0xE0 | (0x20>>3) | (nowrite>>8);
		Pbuffer[(*wpos)+1] = nowrite & 0xFF;
	}
	else
		Pbuffer[*wpos]     = 0x20 | nowrite;
	Pbuffer[(*wpos)+1+tmp] = Psrc[*rpos];
	*wpos += (2+tmp);
	*rpos += (nowrite+1);
}
int Operate40Score(const BYTE *Psrc , int pos , int maxpos , bool isadjust)
{
int tmp1,tmp2;
	tmp1 = Psrc[pos];
	tmp2 = Psrc[pos+1];
	if(tmp1 == tmp2)return 0;
	int i ;
	for(i=0 ; ; i+=2)
	{
		if(i+pos >= maxpos || i+pos+1 >= maxpos){i-=2;break;}
		if(Psrc[i+pos]   != tmp1)break;
		if(Psrc[i+pos+1] != tmp2)break;
	}
	//iA	
	if(isadjust)
		return i-3;
	else 
		return i/2;
}
void Operate40(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos )
{
int nowrite=Operate40Score(Psrc , *rpos , maxpos , false)-1;
int tmp=0;
	if( nowrite > 0x1F)
	{
		if( nowrite > 0x03FF)
		{
			nowrite = 0x03FF;
		}
		tmp = 1;
		Pbuffer[*wpos]     = 0xE0 | (0x40>>3) | (nowrite>>8);
		Pbuffer[(*wpos)+1] = nowrite & 0xFF;
	}
	else
		Pbuffer[*wpos]     = 0x40 | nowrite;
	Pbuffer[(*wpos)+1+tmp] = Psrc[*rpos];
	Pbuffer[(*wpos)+2+tmp] = Psrc[(*rpos)+1];
	*wpos += (3+tmp);
	*rpos += (nowrite+1)*2;
}
int Operate60Score(const BYTE *Psrc , int pos , int maxpos , bool isadjust)
{
BYTE tmp;
	tmp = Psrc[pos];
	int i ;
	for(i=0 ; ; i++)
	{
		if(i+pos >= maxpos){break;}
		if(Psrc[i+pos] != tmp)break;
		tmp++;
	}
	if(isadjust)
		return i-2;
	else 
		return i;
}
void Operate60(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos )
{
int nowrite=Operate60Score(Psrc , *rpos , maxpos , false)-1;
int tmp=0;
	if( nowrite > 0x1F)
	{
		if( nowrite > 0x03FF)
		{
			nowrite = 0x03FF;
		}
		tmp = 1;
		Pbuffer[*wpos]     = 0xE0 | (0x60>>3) | (nowrite>>8);
		Pbuffer[(*wpos)+1] = nowrite & 0xFF;
	}
	else
		Pbuffer[*wpos]     = 0x60 | nowrite;
	Pbuffer[(*wpos)+1+tmp] = Psrc[*rpos];
	*wpos += (2+tmp);
	*rpos += (nowrite+1);
}
int Operate80Score(const BYTE *Psrc , int endpos , int length , int *refpos ,bool isadjust)
{
int maxval;
int maxpos;
	maxval = -1;
	maxpos = 0;
	for(int i=0 ; i<endpos ; i++)
	{
		int q ;
		for(q=0 ;; q++)
		{
			if(endpos+q >= length){break;}
			if(Psrc[i+q]!=Psrc[endpos+q])break;
		}
		//qv
		if(maxval < q)
		{
			maxval = q;
			maxpos = i;
		}
	}
	if(refpos)*refpos = maxpos;
	if(isadjust)
		return maxval-3;
	else 
		return maxval;
}
void Operate80(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos )
{
int tmp=0;
int refpos=0;
int nowrite=Operate80Score(Psrc , *rpos , maxpos , &refpos , false)-1;

	if( nowrite > 0x1F)
	{
		if( nowrite > 0x03FF)
		{
			nowrite = 0x03FF;
		}
		tmp = 1;
		Pbuffer[*wpos]     = 0xE0 | (0x80>>3) | (nowrite>>8);
		Pbuffer[(*wpos)+1] = nowrite & 0xFF;
	}
	else
		Pbuffer[*wpos]     = 0x80 | nowrite;
	Pbuffer[(*wpos)+1+tmp] = refpos / 0x0100;
	Pbuffer[(*wpos)+2+tmp] = refpos % 0x0100;
	*wpos += (3+tmp);
	*rpos += (nowrite+1);
}
//Ƃ肠Aȉ͓xO
int OperateA0Score(const BYTE *Psrc , int endpos , int length , int *refpos ,bool isadjust)
{
	return 0;
}
void OperateA0(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos )
{

}
int OperateC0Score(const BYTE *Psrc , int endpos , int length , int *refpos ,bool isadjust)
{
	return 0;
}
void OperateC0(const BYTE *Psrc , int *rpos , int maxpos , BYTE *Pbuffer , int *wpos )
{

}


//opeɏ]AIɈks
void CmpPart(int ope , const BYTE *Psrc , int *Ppos , int maxpos , BYTE *Pbuffer , int *Pwpos , int val)
{
int i;
int tmp;
	switch(ope)
	{
	case 0x00:
		tmp = 0;
		val --;
		if(val > 0x1F)
		{
			Pbuffer[*Pwpos]   = 0xE0 | (val >> 8);
			Pbuffer[*Pwpos+1] = val & 0xFF;
			tmp = 1;
		}
		else
			Pbuffer[*Pwpos] = val;
		val ++;
		for(i=0 ; i<val ; i++)
			Pbuffer[*Pwpos+i+1+tmp] = Psrc[*Ppos-val+i];
		*Pwpos += (val+1+tmp);
	break;
	case 0x20:
		Operate20( Psrc , Ppos , maxpos , Pbuffer , Pwpos );
	break;
	case 0x40:
		Operate40( Psrc , Ppos , maxpos , Pbuffer , Pwpos );
	break;
	case 0x60:
		Operate60( Psrc , Ppos , maxpos , Pbuffer , Pwpos );
	break;
	case 0x80:
		Operate80( Psrc , Ppos , maxpos , Pbuffer , Pwpos );
	break;
	case 0xA0:
		OperateA0( Psrc , Ppos , maxpos , Pbuffer , Pwpos );
	break;
	case 0xC0:
		OperateC0( Psrc , Ppos , maxpos , Pbuffer , Pwpos );
	break;
	default:
		assert(0);
	}
}

int TwinkleEncode( BYTE **PPout , const BYTE *Psrc , int length , int padding )
{
BYTE *Pbuffer;
	Pbuffer = (BYTE*)malloc(length+length/1024+100);
	//Ƒ傫߂ɊmہAkTCY邱Ƃ̂ŁB
	//A񈳏kJԂƁATCY2049/2048xɂȂ̂ŁAƗ]TڂɊmہB

int bway;
int bval;
int tmp;
int nwcounter;
int wpos;
	nwcounter=0;
	wpos = 0;
	for(int i=0;;)
	{
		//{>͂ǂ
		if(i >= length)
		{
			if(nwcounter)	//񈳏k߂珑
			{
				CmpPart( 0 , Psrc , &i , length , Pbuffer , &wpos , nwcounter );
			}
			//I[M
			Pbuffer[wpos] = 0xFF;
			wpos ++;
			break;
		}
		//xXgEFCƁAxXglij炵
		bway = 0;
		bval = 0;

#define		OPTEST(num)									;\
		tmp = Operate##num##Score(Psrc , i , length);	\
		if(tmp > bval)									\
		{												\
			bval = tmp;									\
			bway = 0x##num;								\
		}
		OPTEST(20);
		OPTEST(40);
		OPTEST(60);
		OPTEST(80);
		OPTEST(A0);
		OPTEST(C0);

		if(bval >= 1)//AkقƔfcc
		{
			//ks
			if( padding )
			{
			//pfBOɒ
				if( bway == 0x20 )//A݂Ώۂ
				{
//int nocon = Operate20Score( Psrc , i , length , false ) ;
					if( padding == 1 )//A񈳏k߂ɂ܂ĂAɂP܂܂
					{
						if( nwcounter && nwcounter != 0x1F )
						{
							nwcounter++ ;
							i++;
							padding = 0 ;
						}
						//Á݂ARoCgȏōŝŖȂ͂cc
					}
					else if( nwcounter )
					{//񈳏k߂Ƃ
//bval̒l܂ŃpfBO\ccɂĂ
						//߂̌ɎĝŁAbval̒l܂ŃpfBOł
						if( padding <= bval )//pfBOI
						{//Ă܂
							if( nwcounter <= 0x1F && nwcounter+padding > 0x1F )
							{
								nwcounter += padding-1 ;
								i += padding-1 ;
								padding = 0 ;
							}
							else
							{
								nwcounter += padding ;
								i += padding ;
								padding = 0 ;
							}
						}
						else
						{
							if(bval)//bval==0͌ł͂肦Ȃ
							{
							//o(bval)pfBO邪APc͕|
								if( padding - bval == 1 )
								{
									if(bval>=2)//ݗʃ[̓_
									{//T߂ɏ
										if( nwcounter <= 0x1F && nwcounter+bval-1 > 0x1F )
										{
										}
										else
										{
											nwcounter+=bval-1;
											i += bval-1;
											padding -= (bval-1);
										}
									}
								}
								else
								{
								//Pc̊댯ȂAۂ菑
									if( nwcounter <= 0x1F && nwcounter+bval > 0x1F )
									{
										nwcounter+=bval-1;
										i+=bval-1;
										padding -= bval-1;
									}
									else
									{
										nwcounter+=bval;
										i+=bval;
										padding -= bval;
									}
								}
							}
						}
					}
					else
					{//񈳏k߂Ƃ
						if( padding <= bval+1 )//pfBOI
						{
							if( nwcounter <= 0x1F && nwcounter+padding-1 > 0x1F )
							{
								nwcounter += padding-2 ;
								i += padding-2 ;
								padding = 0;
							}
							else
							{
								nwcounter += padding-1 ;
								i += padding-1 ;
								padding = 0;
							}
						}
						else
						{
//bval̒l܂ŃpfBO\ccɂĂ
							if( bval )//bval==0͌ł͂肦Ȃǁcc
							{
							//ݖ߂ƁAbval̕
								if( padding - (bval-1) == 1 )//Pĉ͕|
								{
									if( bval != 1 )//PȂށccAݗʂO̓_
									{//T߂ɏ
										if( bval-1 > 0x1F )
										{
											nwcounter += bval-2 ;
											i += bval-2 ;
											padding -= bval-1;
										}
										else
										{
											nwcounter += bval-1 ;
											i += bval-1 ;
											padding -= bval;
										}
									}
								}
								else
								{//ۂ菑
									if( bval > 0x1F )
									{
										nwcounter += bval-1 ;
										i += bval-1 ;
										padding -= bval;
									}
									else
									{
										nwcounter += bval ;
										i += bval ;
										padding -= bval+1;
									}
								}
							}
						}
					}
				}
			}
			if(nwcounter)//񈳏k߂Ȃ珑ācc
			{
				CmpPart( 0 , Psrc , &i , length , Pbuffer , &wpos , nwcounter );
				nwcounter = 0;
			}
			//܂pfBOɃIy[gQÕTCY(1F܂ύXɂ)̑ΉoĂȂłcc
			CmpPart( bway , Psrc , &i , length , Pbuffer , &wpos , bval);
		}
		else
		{
			//񈳏k̂ق悳ȂA񈳏k߂ɓĂ
			nwcounter++;
			i++;
		}
	}
	*PPout = (BYTE*)malloc(wpos);
	memcpy( *PPout , Pbuffer , wpos );
	free(Pbuffer);
	if(padding)return -1;
	return wpos;
}
